unit oCPParameterFile;

interface

uses
  Classes,
  ComCtrls,
  SysUtils,
  XMLDoc,
  XMLIntf,
  oCP_Exceptions,
  oCPTypes,
  oInterfaces;

type
  TCPParameterFile = class(TInterfacedObject, ICPParameterFile)
  private
    fDBConnection: IDBConnection;
    fApplicationKey: string;
    fName: string;
    fXMLDoc: IXMLDocument;
    fXMLRec: IXMLNode;
    fValues: array[TCPParamKey] of TStringList;
  protected
    function getApplicationKey: string; stdcall;
    procedure setApplicationKey(aKey: string); stdcall;
    procedure LoadValues(aKey: TCPParamKey; aList: TStrings); stdcall;
    procedure ClearValues(aKey: TCPParamKey); stdcall;

    function ReadDateTime(aKey: TCPParamKey; aParam: string; aDefault: TDateTime): TDateTime; stdcall;
    function ReadBoolean(aKey: TCPParamKey; aParam: string; aDefault: boolean = False): Boolean; stdcall;
    function ReadInteger(aKey: TCPParamKey; aParam: string; aDefault: integer = 0): integer; stdcall;
    function ReadDouble(aKey: TCPParamKey; aParam: string; aDefault: double = 0.0): double; stdcall;
    function ReadString(aKey: TCPParamKey; aParam: string; aDefault: string = ''): string; overload; stdcall;
    function ReadString(aKey: string; aParam: string; aDefault: string = ''): string; overload; stdcall;

    procedure WriteDateTime(aKey: TCPParamKey; aParam: string; aValue: TDateTime); stdcall;
    procedure WriteBoolean(aKey: TCPParamKey; aParam: string; aValue: boolean); stdcall;
    procedure WriteInteger(aKey: TCPParamKey; aParam: string; aValue: integer); stdcall;
    procedure WriteDouble(aKey: TCPParamKey; aParam: string; aValue: double); stdcall;
    procedure WriteString(aKey: TCPParamKey; aParam: string; aValue: string); overload; stdcall;
    procedure WriteString(aKey: string; aParam: string; aValue: string); overload; stdcall;

  public
    constructor Create(aDBConnection: IDBConnection; aApplicationKey: string = 'MD PARAMETERS'; aName: string = '');
    destructor Destroy; override;
  end;

implementation

uses
  uCPXML;

{ TCliOParameterFile }

constructor TCPParameterFile.Create(aDBConnection: IDBConnection; aApplicationKey: string = 'MD PARAMETERS'; aName: string = '');
begin
  inherited Create;
  fDBConnection := aDBConnection;
  fXMLDoc := NewXMLDocument;
  fApplicationKey := aApplicationKey;

  fValues[pkSystem] := TStringList.Create;
  fValues[pkDivision] := TStringList.Create;
  fValues[pkUser] := TStringList.Create;

  if aName = '' then
    fName := Format('Parameter File [%s]', [aApplicationKey])
  else
    fName := aName;
end;

destructor TCPParameterFile.Destroy;
begin
  FreeAndNil(fValues[pkSystem]);
  FreeAndNil(fValues[pkDivision]);
  FreeAndNil(fValues[pkUser]);
  fXMLDoc := nil;
  fDBConnection := nil;
  inherited;
end;

procedure TCPParameterFile.ClearValues(aKey: TCPParamKey);
begin
  try
    if not fDBConnection.ExecCommand('ClearParameterSettings', [CP_PARAM_KEY_NAME[aKey], fApplicationKey]) then
      raise Exception.CreateFmt('Error clearing parameter settings ''%s''.', [fApplicationKey]);
    if fDBConnection.ExecResults[0] <> '1' then
      raise Exception.CreateFmt('Error clearing parameter settings: ''%s''.', [fDBConnection.ExecResults.Text]);
  except
    raise
  end;
end;

function TCPParameterFile.getApplicationKey: string;
begin
  Result := fApplicationKey;
end;

procedure TCPParameterFile.LoadValues(aKey: TCPParamKey; aList: TStrings);
begin
  try
    aList.Clear;
    if not fDBConnection.ExecQuery('GetParameterList', [CP_PARAM_KEY_NAME[aKey], fApplicationKey], fXMLDoc) then
      raise Exception.Create('Error getting list of parameters');
    fXMLRec := XMLDocumentFirstRecord(fXMLDoc);
    while Assigned(fXMLRec) do
      try
        aList.Values[XMLValueAsStr(fXMLRec, 'NAME')] := XMLValueAsStr(fXMLRec, 'VALUE');
      finally
        fXMLRec := XMLDocumentNextRecord(fXMLRec);
      end;
  except
    raise;
  end;
end;

function TCPParameterFile.ReadBoolean(aKey: TCPParamKey; aParam: string; aDefault: boolean): Boolean;
begin
  try
    Result := (ReadString(aKey, aParam, '0') = '1');
  except
    Result := aDefault;
  end;
end;

function TCPParameterFile.ReadDateTime(aKey: TCPParamKey; aParam: string; aDefault: TDateTime): TDateTime;
var
  x: string;
  yy, mm, dd, h, m, s: word;
begin
  {YYYY-MM-DD HH:MM:SS}
  try
    x := ReadString(aKey, aParam);
    yy := StrToInt(Copy(x, 1, 4));
    mm := StrToInt(Copy(x, 6, 2));
    dd := StrToInt(Copy(x, 9, 2));
    h := StrToInt(Copy(x, 12, 2));
    m := StrToInt(Copy(x, 15, 2));
    s := StrToInt(Copy(x, 18, 2));
    Result := EncodeDate(yy, mm, dd) + EncodeTime(h, m, s, 0);
  except
    Result := aDefault;
  end;
end;

function TCPParameterFile.ReadDouble(aKey: TCPParamKey; aParam: string; aDefault: double): double;
begin
  try
    Result := StrToFloatDef(ReadString(aKey, aParam, ''), aDefault);
  except
    Result := aDefault;
  end;
end;

function TCPParameterFile.ReadInteger(aKey: TCPParamKey; aParam: string; aDefault: integer): integer;
begin
{ 2FA
  try
    Result := StrToInt(ReadString(aKey, aParam, ''));
  except
    Result := aDefault;
  end;
}
  Result := StrToIntDef(ReadString(aKey, aParam, ''),aDefault);
end;

function TCPParameterFile.ReadString(aKey: TCPParamKey; aParam, aDefault: string): string;
begin
  Result := ReadString(CP_PARAM_KEY_NAME[aKey], aParam, aDefault);
end;

function TCPParameterFile.ReadString(aKey: string; aParam: string; aDefault: string): string;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
begin
  aXMLDoc := NewXMLDocument;
  try
    if not fDBConnection.ExecQuery('GetParameterValue', [aKey, fApplicationKey, aParam], aXMLDoc) then
      raise ECPLoadFailure.CreateFmt('Error loading parameter value for parameter ''%s''.', [aParam]);
    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    if Assigned(aXMLRec) then
      Result := XMLValueAsStr(aXMLRec, 'VALUE', aDefault)
    else
      Result := aDefault;
  except
    raise;
  end;
end;

procedure TCPParameterFile.setApplicationKey(aKey: string);
begin
  fApplicationKey := aKey;
  {
  try
    if not fDBConnection.ExecCommand('OpenParameterSettings', [aKey]) then
      raise Exception.CreateFmt('Error opening parameter settings ''%s''.', [aKey]);
    if fDBConnection.ExecResults[0] <> '1' then
      raise Exception.CreateFmt('Error accessing parameter settings: %s', [fDBConnection.ExecResults.Text]);
    fApplicationKey := aKey;
  except
    raise
  end;
  }
end;

procedure TCPParameterFile.WriteBoolean(aKey: TCPParamKey; aParam: string; aValue: boolean);
begin
  case aValue of
    True: WriteInteger(aKey, aParam, 1);
    False: WriteInteger(aKey, aParam, 0);
  end;
end;

procedure TCPParameterFile.WriteDateTime(aKey: TCPParamKey; aParam: string; aValue: TDateTime);
begin
  WriteString(aKey, aParam, FormatDateTime('YYYY-MM-DD HH:MM:SS', aValue));
end;

procedure TCPParameterFile.WriteDouble(aKey: TCPParamKey; aParam: string; aValue: double);
begin
  WriteString(aKey, aParam, FloatToStr(aValue));
end;

procedure TCPParameterFile.WriteInteger(aKey: TCPParamKey; aParam: string; aValue: integer);
begin
  WriteString(aKey, aParam, IntToStr(aValue));
end;

procedure TCPParameterFile.WriteString(aKey, aParam, aValue: string);
begin
  try
    if aValue = '' then
      aValue := '@';
    if not fDBConnection.ExecCommand('UpdateParameterValue', [aKey, fApplicationKey, aParam, aValue]) then
      raise ECPSaveFailure.CreateFmt('Error saving parameter value for parameter ''%s''.', [aParam]);
  except
    raise;
  end;
end;

procedure TCPParameterFile.WriteString(aKey: TCPParamKey; aParam, aValue: string);
begin
  WriteString(CP_PARAM_KEY_NAME[aKey], aParam, aValue);
end;

end.

