unit oCliOLoader;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 9 $  $Modtime: 11/16/10 10:15a $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Observation Loader.  Gets observations from DBConnection.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCliOLoader.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  Grids,
  Dialogs,
  Graphics,
  SysUtils,
  oCliO,
  oCliOTypes,
  oInterfaces,
  oObservation,
  oTerm,
  Variants,
  XMLIntf,
  XMLDoc;

type
  TCliOLoader = class(TInterfacedObject, ICliOLoader)
  private
    fDBConnection: IDBConnection;
    fLoadErrors: TStringList;
    fLoadAtATime: integer;
    fTerminology: ITerminology;
    fOnNotifyControllerStatus: TNotifyOnProgress;

    procedure NotifyOnProgress(const aMsg: string; aProgress, aProgressMax: integer);
    function LoadList(aList: ICliOList; aQueryName: string; aParams: array of variant; aLoadMessage: string = 'Retrieving Observations'): boolean;

    {ICliOLoader interface}
    function CreateNewList: ICliOList;
    function CreateNewHL7Log: ICliOHL7Log;

    function GetObservationByID(aID: string): IObservation; stdcall;

    procedure LoadObservations(aList: ICliOList; aPatientID: string; aStartDateTime, aStopDateTime: TDateTime); stdcall;
    procedure LoadUnverifedObservations(aList: ICliOList; aPatientID: string); stdcall;
    procedure LoadObservationsByStatus(aList: ICliOList; aPatientID: string; aStartDateTime, aStopDateTime: TDateTime; aStatus: TObservationRecordStatusSet); stdcall;
    procedure LoadAuditLog(aList: TStrings; aObservationID: string); stdcall;
    procedure SetLoadAtATime(aValue: integer); stdcall;
    procedure SetOnNotify(aOnNotifyOnProgress: TNotifyOnProgress); stdcall;
  public
    constructor Create(aTerminology: ITerminology; aDBConnection: IDBConnection);
    destructor Destroy; override;

    {Refresh Error mgmt methods}
    function GetRefreshErrorCount: Integer;
    function GetRefreshErrors(var aStringList: TStringList): Boolean;

    {External calls to manipulate the loader}
    procedure GetIGridItemList(aList: IInterfaceList); stdcall;
  end;

implementation

uses
  iniFiles,
  DateUtils,
  Controls,
  iCliOGrid,
  oCliOList,
  oCP_Exceptions,
  //oObservation_View_Term,
  oObservation_Audit,
  oCliOHL7Log,
  uCPXML,
  Masks;

type
  TObservationColumns = (ocName, ocDateTimeObserved, ocValue, ocStatus);

const
  OBSERVATION_COLUMNS: array[TObservationColumns] of string = ('NAME', 'Date/Time Observed', 'Value', 'Status');
  OBSERVATION_COLUMN_WIDTHS: array[TObservationColumns] of integer = (150, 150, 200, 100);

  {------------------------------------------------------------------------------}
  {--------------------------- TObservation_Loader ------------------------------}
  {------------------------------------------------------------------------------}

constructor TCliOLoader.Create(aTerminology: ITerminology; aDBConnection: IDBConnection);
begin
  inherited Create;
  fLoadErrors := TStringList.Create;
  fLoadAtATime := 20;
  fTerminology := aTerminology;
  fDBConnection := aDBConnection;
  fOnNotifyControllerStatus := NotifyOnProgress;
end;

destructor TCliOLoader.Destroy;
begin
  FreeAndNil(fLoadErrors);
  inherited;
end;

function TCliOLoader.CreateNewHL7Log: ICliOHL7Log;
begin
  NewCliOHL7Log(fDBConnection).GetInterface(ICliOHL7Log, Result);
end;

function TCliOLoader.CreateNewList: ICliOList;
begin
  NewCliOList.GetInterface(ICliOList, Result);
end;

function TCliOLoader.getRefreshErrorCount: Integer;
begin
  Result := fLoadErrors.Count;
end;

function TCliOLoader.getRefreshErrors(var aStringList: TStringList): Boolean;
begin
  aStringList.Clear;
  aStringList.AddStrings(fLoadErrors);
  Result := True;
end;

procedure TCliOLoader.GetIGridItemList(aList: IInterfaceList);
begin
  (*
  {Clear the list of any existing 'stuff'}
  aList.Clear;

  for i := 0 to fInterfacedList.ObservationCount - 1 do
    if fInterfacedList.Observation(i).QueryInterface(IGridItem, aGridItem) = 0 then
      aList.Add(aGridItem);
  *)
end;

function TCliOLoader.GetObservationByID(aID: string): IObservation;
var
  aDoc: IXMLDocument;
  aRec: IXMLNode;
  aChild: IObservation;
begin
  try
    aDoc := NewXMLDocument;

    if MatchesMask(aID, 'GMRV*') then
    begin
      if fDBConnection.ExecQuery('GetVitalsByID', [aID], aDoc) then
        begin
          aRec := XMLDocumentFirstRecord(aDoc);
          Result := NewObservation(aRec, fTerminology, true);
          aRec := XMLDocumentNextRecord(aRec);
          while Assigned(aRec) do
            try
              aChild := NewObservation(aRec, fTerminology, true);
              aChild.setParent(Result);
            finally
              aRec := XMLDocumentNextRecord(aRec);
            end;
        end
    end
    else
    begin
      if fDBConnection.ExecQuery('GetObservationByID', [aID], aDoc) then
        begin
          aRec := XMLDocumentFirstRecord(aDoc);
          Result := NewObservation(aRec, fTerminology);
          aRec := XMLDocumentNextRecord(aRec);
          while Assigned(aRec) do
            try
              aChild := NewObservation(aRec, fTerminology);
              aChild.setParent(Result);
            finally
              aRec := XMLDocumentNextRecord(aRec);
            end;
        end
    end;
  except
    Result := nil;
  end;
end;

procedure TCliOLoader.LoadUnverifedObservations(aList: ICliOList; aPatientID: string);
begin
  LoadList(aList, 'GetObservationsByStatus', [aPatientID, 0]);
end;

procedure TCliOLoader.NotifyOnProgress(const aMsg: string; aProgress, aProgressMax: integer);
begin
  // No nil pointers allowed :)
end;

procedure TCliOLoader.SetLoadAtATime(aValue: integer);
begin
  if aValue > 0 then
    fLoadAtATime := aValue;
end;

procedure TCliOLoader.SetOnNotify(aOnNotifyOnProgress: TNotifyOnProgress);
begin
  fOnNotifyControllerStatus := aOnNotifyOnProgress;
end;

procedure TCliOLoader.LoadObservationsByStatus(aList: ICliOList; aPatientID: string; aStartDateTime, aStopDateTime: TDateTime; aStatus: TObservationRecordStatusSet);
var
  aStat: TObservationRecordStatus;
  aTmpList : ICliOList;
  I: Integer;

begin
  aTmpList := CreateNewList;
  aList.Clear;

  for aStat := Low(TObservationRecordStatus) to High(TObservationRecordStatus) do
    if aStat in aStatus then
      begin
        LoadList(aTmpList, 'GetObservationsByStatus',
          [aPatientID, aStartDateTime, aStopDateTime, aStat],
          Format('Retrieving ''%s'' Observations', [OBSERVATION_STATUS[aStat]]));
        aTmpList.CopyTo(aList);
      end;

  aTmpList.Clear;

  LoadList(aTmpList, 'GetVitals', [aPatientID, aStartDateTime, aStopDateTime]);

  for I := 0 to aTmpList.ObservationCount - 1 do
  begin
    if aTmpList.Observation(I).getStatus in aStatus then
      aList.Add(aTmpList.Observation(i));
  end;
end;

function TCliOLoader.LoadList(aList: ICliOList; aQueryName: string; aParams: array of variant; aLoadMessage: string = 'Retrieving Observations'): boolean;
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
  aObservation: IObservation;
  aObsList: TStringList;
  aVitalsList: TStringList;
  aObsParam: array of variant;
  aObsCount: integer;
  aObsDone: integer;
  i: integer;
  aID: string;
begin
  aList.Clear;
  fLoadErrors.Clear;
  aXMLDoc := NewXMLDocument;
  aObsList := TStringList.Create; {Holds the list of ID's to load after first call}
  aVitalsList := TStringList.Create; { Holds Vitals ID's}
  try
  SetLength(aObsParam, fLoadAtATime);

  {Call out the first query to get the ID's of the observations that we need}
  if not fDBConnection.ExecQuery(aQueryName, aParams, aXMLDoc) then
    begin
      fLoadErrors.Add('Error loading observations.');
      Result := False;
      Exit;
    end
  else
    begin
      aXMLDoc.Active := True;
      aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
      while Assigned(aXMLNode) do
        try
          aID := XMLValueAsStr(aXMLNode, 'OBS_ID');
          if MatchesMask(aID, '{GMRV*') then
            aVitalsList.Add(aID)
          else
            aObsList.Add(aID);
        finally
          aXMLNode := XMLDocumentNextRecord(aXMLNode);
        end;
    end;

  {Needed to calculate the number of calls left to make}
  aObsCount := aVitalsList.Count;
  aObsDone := 0;

  //****************************************************************************
  //  Ok, aVitalsList contains 0..n strings equating to the ID's we need from
  //  the db. Next we load 'em up in sets and call 'em out
  //  The const LOAD_AT_A_TIME contains the count to get each time
  //****************************************************************************

  while aVitalsList.Count > 0 do
    begin
      {Load obs id's into the param}
      for i := Low(aObsParam) to High(aObsParam) do
        if aVitalsList.Count > 0 then
          begin
            aObsParam[i] := aVitalsList[0];
            aVitalsList.Delete(0);
            inc(aObsDone);
          end
        else
          aObsParam[i] := '';

      fOnNotifyControllerStatus('Loading', aObsDone, aObsCount);

      {Load real observations}
      if not fDBConnection.ExecQuery('GetVitalsByID', aObsParam, aXMLDoc) then
        begin
          fLoadErrors.Add('Error loading observations.');
          Result := False;
          Exit;
        end
      else
        begin
          aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
          while aXMLNode <> nil do
            try
              aObservation := NewObservation(aXMLNode, fTerminology, true);
              if Assigned(aObservation) then
                aList.Add(aObservation)
              else
                fLoadErrors.Add('Unable to load :' + #13 + aXMLNode.XML);
              aXMLNode := XMLDocumentNextRecord(aXMLNode);
            except
              on E: ECPException do
                begin
                  fLoadErrors.Add(e.Message);
                  aXMLNode := XMLDocumentNextRecord(aXMLNode);
                end
            else
              raise;
            end;
        end;
    end;


  {Needed to calculate the number of calls left to make}
  aObsCount := aObsList.Count;
  aObsDone := 0;

  //****************************************************************************
  //  Ok, aObsList contains 0..n strings equating to the ID's we need from
  //  the db. Next we load 'em up in sets and call 'em out
  //  The const LOAD_AT_A_TIME contains the count to get each time
  //****************************************************************************

  while aObsList.Count > 0 do
    begin
      {Load obs id's into the param}
      for i := Low(aObsParam) to High(aObsParam) do
        if aObsList.Count > 0 then
          begin
            aObsParam[i] := aObsList[0];
            aObsList.Delete(0);
            inc(aObsDone);
          end
        else
          aObsParam[i] := '';

      fOnNotifyControllerStatus('Loading', aObsDone, aObsCount);

      {Load real observations}
      if not fDBConnection.ExecQuery('GetObservationByID', aObsParam, aXMLDoc) then
        begin
          fLoadErrors.Add('Error loading observations.');
          Result := False;
          Exit;
        end
      else
        begin
          aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
          while aXMLNode <> nil do
            try
              aObservation := NewObservation(aXMLNode, fTerminology);
              if Assigned(aObservation) then
                aList.Add(aObservation)
              else
                fLoadErrors.Add('Unable to load :' + #13 + aXMLNode.XML);
              aXMLNode := XMLDocumentNextRecord(aXMLNode);
            except
              on E: ECPException do
                begin
                  fLoadErrors.Add(e.Message);
                  aXMLNode := XMLDocumentNextRecord(aXMLNode);
                end
            else
              raise;
            end;
        end;
    end;

  {Link up the parent child observations}
  aObservation := aList.FirstObservation;
  while aObservation <> nil do
    begin
      if aObservation.getParentID <> '' then
        begin
          if aList.Observation(aObservation.getParentID) <> nil then
            aObservation.setParent(aList.Observation(aObservation.getParentID))
          else
            fLoadErrors.Add('Missing a parent observation.');
        end;
      aObservation := aList.NextObservation(aObservation);
    end;

  Result := (fLoadErrors.Count = 0);

  if Result then
    aList.SortByDateTime
  else
    aList.Clear;

  fOnNotifyControllerStatus('', 0, 0);
  aXMLDoc := nil;

  finally
    FreeAndNil(aObsList);
    FreeAndNil(aVitalsList);
  end;
end;

procedure TCliOLoader.LoadObservations(aList: ICliOList; aPatientID: string; aStartDateTime, aStopDateTime: TDateTime);
var
  TempList: ICliOList;
begin
  TempList := CreateNewList;
  LoadList(aList, 'GetObservationsByDate', [aPatientID, aStartDateTime, aStopDateTime]);
  LoadList(TempList, 'GetVitals', [aPatientID, aStartDateTime, aStopDateTime]);
  TempList.CopyTo(aList);
end;

procedure TCliOLoader.LoadAuditLog(aList: TStrings; aObservationID: string);
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aAudit: TObservation_Audit;
begin
  aXMLDoc := NewXMLDocument;
  if fDBConnection.ExecQuery('GetObservationAuditLog', [aObservationID], aXMLDoc) then
    begin
      aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
      while Assigned(aXMLRec) do
        begin
          aAudit := NewAuditEntry(aXMLRec);
          if Assigned(aAudit) then
            aList.AddObject(DateTimeToStr(aAudit.AuditDateTime), aAudit);
          aXMLRec := XMLDocumentNextRecord(aXMLRec);
        end;
    end;
  aXMLDoc := nil;
end;

end.

