unit oCliOReportGenerator;

interface

uses
  Classes,
  ComCtrls,
  Controls,
  Dialogs,
  Graphics,
  SysUtils,
  Types,
  Printers,
  oCliO,
  oInterfaces;

type
  TCliOReportGenerator = class(TManagedInterfacedObject, ICliOReportGenerator)
  private
    fColumns: TStringList;

    fPageRect: TRect;

    fHeaderRect: TRect;
    fHeaderFont: TFont;
    fHeaderCentered: boolean;

    fDataRect: TRect;
    fDataFont: TFont;

    fFooterRect: TRect;
    fFooterFont: TFont;

    fPrintDateTime: string;
    fTitle: string;
    fSubTitles: TStringList;
    fLineCount: integer;

    xBorder: integer;
    yBorder: integer;

    function getPrinterSetup: boolean;
    procedure Initialize(aListView: TListView = nil);
    procedure BeginDoc;
    procedure EndDoc;
    procedure DrawHeader;
    procedure DrawFooter;
    procedure DrawLine(aLine: integer; aText: string);
    procedure DrawCell(aLine: integer; aCol: integer; aText: string);
    procedure FrameCurrentPage;

    {Interface Methods}
    procedure PrintTestPage; stdcall;
    procedure PrintListView(aListView: TListView); stdcall;
    procedure SetTitle(aTitle: string); stdcall;
    procedure SetPatient(aPatient: IPatient); stdcall;
    procedure AddSubTitle(aSubTitle: string); stdcall;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

type
  TCliOReportColumn = class(TObject)
    fLeftCol: integer;
    fWidthPercent: Double;
    fWidthPixels: integer;
  end;

{ TCliOPrintout }

procedure TCliOReportGenerator.AddSubTitle(aSubTitle: string);
begin
  fSubTitles.Add(aSubTitle);
end;

procedure TCliOReportGenerator.BeginDoc;
begin
  Printer.BeginDoc;
end;

constructor TCliOReportGenerator.Create;
begin
  inherited;
  fColumns := TStringList.Create;
  fSubTitles := TStringList.Create;
end;

destructor TCliOReportGenerator.Destroy;
begin
  while fColumns.Count > 0 do
    begin
      TCliOReportColumn(fColumns.Objects[0]).Free;
      fColumns.Delete(0);
    end;
  FreeAndNil(fColumns);
  FreeAndNil(fSubTitles);
  FreeAndNil(fHeaderFont);
  FreeAndNil(fDataFont);
  FreeAndNil(fFooterFont);
  inherited;
end;

procedure TCliOReportGenerator.DrawCell(aLine, aCol: integer; aText: string);
var
  aCliOReportColumn: TCliOReportColumn;
  aPageLine: integer;
begin
  if aText <> '' then
    with Printer.Canvas do
      begin
        {
        if aLine > fLineCount then
          aPageLine := trunc(aLine / fLineCount)
        else
          aPageLine := aLine;
        }

        aPageLine := aLine - ((Printer.PageNumber - 1) * fLineCount);

        Font := fDataFont;
        aCliOReportColumn := TCliOReportColumn(fColumns.Objects[aCol]);
        if TextWidth(aText) > aCliOReportColumn.fWidthPixels then
          while TextWidth(Format('%s...', [aText])) > aCliOReportColumn.fWidthPixels do
            aText := Copy(aText, 1, Length(aText) - 1);
        TextOut(
          aCliOReportColumn.fLeftCol + (xBorder * 2),
          fDataRect.Top + ((aPageLine - 1) * TextHeight('|')),
          aText);
      end;
end;

procedure TCliOReportGenerator.DrawFooter;
var
  aPageNum: string;
begin
  aPageNum := Format('Page: %d', [Printer.PageNumber]);
  with Printer.Canvas do
    begin
      Font := fFooterFont;
      TextOut(fFooterRect.Left, fFooterRect.Top, fPrintDateTime);
      TextOut(fFooterRect.Right - TextWidth(aPageNum + '  '), fFooterRect.Top, aPageNum);
    end;
end;

procedure TCliOReportGenerator.DrawHeader;
var
  i: integer;
begin
  with Printer.Canvas do
    begin
      FillRect(fHeaderRect);
      Font := fHeaderFont;
      if fHeaderCentered then
        begin
          TextOut(
            fHeaderRect.Left + (fHeaderRect.Right - fHeaderRect.Left - TextWidth(fTitle)) div 2,
            fHeaderRect.Top,
            fTitle);

          Font.Style := Font.Style - [fsBold];

          for i := 0 to fSubTitles.Count - 1 do
            TextOut(
              fHeaderRect.Left + (fHeaderRect.Right - fHeaderRect.Left - TextWidth(fSubTitles[i])) div 2,
              fHeaderRect.Top + (TextHeight('|') * (i + 1)),
              fSubTitles[i]);
        end
      else
        begin
          TextOut(
            fHeaderRect.Left,
            fHeaderRect.Top,
            fTitle);

          Font.Style := Font.Style - [fsBold];

          for i := 0 to fSubTitles.Count - 1 do
            TextOut(
              fHeaderRect.Left,
              fHeaderRect.Top + (TextHeight('|') * (i + 1)),
              fSubTitles[i]);
        end;

      if fColumns.Count > 0 then
        begin
          Font := fDataFont;
          Pen.Width := 4;
          MoveTo(fPageRect.Left, fHeaderRect.Bottom - TextHeight('|') - yBorder);
          LineTo(fPageRect.Right, fHeaderRect.Bottom - TextHeight('|') - yBorder);
          Pen.Width := 1;
          for i := 0 to fColumns.Count - 1 do
            with TCliOReportColumn(fColumns.Objects[i]) do
              begin
                TextOut(
                  fLeftCol + (xBorder * 2),
                  fHeaderRect.Bottom - TextHeight('|'), // - (yBorder div 2),
                  fColumns[i]);

                if i > 0 then
                  begin
                    Pen.Color := clGray;
                    MoveTo(fLeftCol, fHeaderRect.Bottom - TextHeight('|') - yBorder);
                    LineTo(fLeftCol, fDataRect.Bottom + yBorder);
                    Pen.Color := clBlack;
                  end;
              end;
        end;
    end;
end;

procedure TCliOReportGenerator.DrawLine(aLine: integer; aText: string);
begin
  with Printer.Canvas do
    begin
      Font := fDataFont;
      TextOut(
        fDataRect.Left,
        fDataRect.Top + ((aLine - 1) * TextHeight('|')),
        Format('%0.4d. %s', [aLine, aText]));
    end;
end;

procedure TCliOReportGenerator.EndDoc;
begin
  Printer.EndDoc;
end;

procedure TCliOReportGenerator.FrameCurrentPage;
begin
  //if Printer.PageNumber > 1 then
  //  Printer.NewPage;

  with Printer.Canvas do
    begin
      Pen.Width := 4;
      Rectangle(fPageRect);

      MoveTo(fPageRect.Left, fHeaderRect.Bottom + yBorder);
      LineTo(fPageRect.Right, fHeaderRect.Bottom + yBorder);

      MoveTo(fPageRect.Left, fDataRect.Bottom + yBorder);
      LineTo(fPageRect.Right, fDataRect.Bottom + yBorder);

      Pen.Width := 1;

      DrawHeader;
      DrawFooter;

      MoveTo(fDataRect.Left, fDataRect.Top);
      Font := fDataFont;
    end;
end;

function TCliOReportGenerator.getPrinterSetup: boolean;
begin
  with TPrinterSetupDialog.Create(nil) do
    try
      Result := Execute;
    finally
      Free;
    end;
end;

procedure TCliOReportGenerator.Initialize(aListView: TListView = nil);
var
  aCol: TCliOReportColumn;
  i, j: integer;
begin
  fPrintDateTime := 'Date Time: ' + DateTimeToStr(Now);

  BeginDoc;

  Printer.Canvas.Pen.Width := 1;
  Printer.Canvas.Font.Name := 'Tahoma';

  fHeaderFont := TFont.Create;
  fHeaderFont.Assign(Printer.Canvas.Font);
  fHeaderFont.Size := 12;
  fHeaderFont.Style := [fsBold];

  fFooterFont := TFont.Create;
  fFooterFont.Assign(Printer.Canvas.Font);
  fFooterFont.Size := 8;
  fFooterFont.Style := [];

  fDataFont := TFont.Create;
  fDataFont.Assign(Printer.Canvas.Font);
  fDataFont.Size := 9;
  fDataFont.Style := [];

  xBorder := trunc(Printer.PageWidth * 0.005);
  yBorder := trunc(Printer.PageHeight * 0.005);

  fPageRect := Rect(
    trunc(Printer.PageWidth * 0.02),
    trunc(Printer.PageHeight * 0.02),
    Printer.PageWidth - trunc(Printer.PageWidth * 0.02),
    Printer.PageHeight - trunc(Printer.PageHeight * 0.02));

  Printer.Canvas.Font := fHeaderFont;
  fHeaderRect := Rect(
    fPageRect.Left + xBorder,
    fPageRect.Top + yBorder,
    fPageRect.Right - xBorder,
    fPageRect.Top + (Printer.Canvas.TextHeight('|') * (fSubTitles.Count + 1)) + (yBorder * 2));

  {Check for a subheader}
  if Assigned(aListView) then
    begin
      Printer.Canvas.Font := fDataFont;
      fHeaderRect.Bottom := fHeaderRect.Bottom + Printer.Canvas.TextHeight('|') + xBorder;
    end;

  Printer.Canvas.Font := fFooterFont;
  fFooterRect := Rect(
    fPageRect.Left + xBorder,
    fPageRect.Bottom - Printer.Canvas.TextHeight('|') - (yBorder * 2),
    fPageRect.Right - xBorder,
    fPageRect.Bottom - yBorder);

  Printer.Canvas.Font := fDataFont;
  fDataRect := Rect(
    fPageRect.Left + xBorder,
    fHeaderRect.Bottom + (yBorder * 2),
    fPageRect.Right - xBorder,
    fFooterRect.Top - (yBorder * 2));

  fLineCount := (fDataRect.Bottom - fDataRect.Top - Printer.Canvas.TextHeight('|')) div Printer.Canvas.TextHeight('|');

  if Assigned(aListView) then
    for i := 0 to aListView.Columns.Count - 1 do
      begin
        aCol := TCliOReportColumn.Create;

        {Assign the left column}
        if i = 0 then
          aCol.fLeftCol := fDataRect.Left
        else
          aCol.fLeftCol := TCliOReportColumn(fColumns.Objects[i - 1]).fLeftCol +
            TCliOReportColumn(fColumns.Objects[i - 1]).fWidthPixels +
            (xBorder * 2);

        aCol.fWidthPixels := Printer.Canvas.TextWidth(aListView.Columns[i].Caption) + (xBorder * 2);

        for j := 0 to aListView.Items.Count - 1 do
          case i of
            0:
              begin
                if aCol.fWidthPixels <
                  (Printer.Canvas.TextWidth(aListView.Items[j].Caption) + (xBorder * 2)) then
                  aCol.fWidthPixels := Printer.Canvas.TextWidth(aListView.Items[j].Caption) + (xBorder * 2);
              end;
          else
            begin
              if aListView.Items[j].SubItems.Count >= i then
                if aCol.fWidthPixels <
                  (Printer.Canvas.TextWidth(aListView.Items[j].SubItems[i - 1]) + (xBorder * 2)) then
                  aCol.fWidthPixels := Printer.Canvas.TextWidth(aListView.Items[j].SubItems[i - 1]) + (xBorder * 2);
            end;
          end;
        fColumns.AddObject(aListView.Columns[i].Caption, aCol);
      end;

  FrameCurrentPage;
end;

procedure TCliOReportGenerator.PrintListView(aListView: TListView);
var
  i: integer;
  j: integer;
begin
  fHeaderCentered := False;
  if getPrinterSetup then
    try
      Initialize(aListView);

      for i := 0 to aListView.Items.Count - 1 do
        begin
          DrawCell(i + 1, 0, aListView.Items[i].Caption);
          for j := 0 to aListView.Items[i].SubItems.Count - 1 do
            DrawCell(i + 1, j + 1, aListView.Items[i].SubItems[j]);
          if (frac((i + 1) / fLineCount) = 0) and (i < (aListView.Items.Count - 2)) then
            begin
              Printer.NewPage;
              FrameCurrentPage;
            end;
        end;
    finally
      EndDoc;
    end;
end;

procedure TCliOReportGenerator.PrintTestPage;
var
  i: integer;
begin
  SetTitle('Test of the ICliOPrintout Interface');
  fSubTitles.Clear;
  AddSubTitle('Patient: SIMPSON,HOMER J  Sex: M  Age: 42');
  AddSubTitle('SSN: 666-66-6666  DOB: 06/24/1964');
  fHeaderCentered := False;

  if getPrinterSetup then
    try
      Initialize;
      for i := 1 to 10 do
        DrawLine(i, 'sample text.');
    finally
      EndDoc;
    end;
end;

procedure TCliOReportGenerator.SetPatient(aPatient: IPatient);
begin
  fSubTitles.Clear;
  AddSubTitle(Format('Patient: %s  Sex: %s  Age: %d', [aPatient.getName, aPatient.getSex, aPatient.getAge]));
  AddSubTitle(Format('SSN: %s  DOB: %s', [aPatient.getSSN, FormatDateTime('mm/dd/yyyy', aPatient.getDOB)]));
end;

procedure TCliOReportGenerator.SetTitle(aTitle: string);
begin
  fTitle := aTitle;
end;

end.

