unit oObservation;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 14 $  $Modtime: 1/05/11 3:16p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Primary object for management of observations.
*
*       Notes:        Observations external to the TDBController_Observations
*                     should ALWAYS be accessed via the IObservation interface.
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oObservation.pas $
================================================================================
}
interface

uses
  Dialogs,
  Classes,
  ComCtrls,
  SysUtils,
  oInterfaces,
  iCliOGrid,
  oTerm,
  oCliO,
  oCliOTypes,
  Graphics,
  Variants,
  XMLintf;

type
  TObservation = class(TInterfacedObject, IObservation, IGridItem, ICliOReportItem)
  protected
    fID: string;
    fBattery_ID: string;
    fPatient_ID: string;
    fPatient_Name: string;
    fObserved_By: string;
    fObserved_By_ID: string;
    fObserved_By_Initials: string;
    fObserved_Date_Time: TDateTime;
    fFacility_Location_ID: string;
    fHospital_Location_ID: string;

    fNullFlavor: Integer;

    fEntered_By: string;
    fEntered_By_ID: string;
    fEntered_Date_Time: TDateTime;
    fComment: string;

    fSource_Comments: string;
    fSource_DataItemID: string;
    fSource_ID: string;
    fSource_Version: string;
    fStatus_Record: TObservationRecordStatus;
    fRange: TObservationRange;

    fParent: IObservation;
    fParent_ID: string;

    fQualifiers: array[TTermType] of ITerm;
    fSets: TStringList;

    fSValue: Variant; {The Big One!!!}

    fCorrection_For_ID: string;
    fAudit_Exists: boolean;

    fChildren: IInterfaceList;

    fReadOnly: Boolean;

    {ICliOReportItem}
    procedure AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean = False; aList: ICliOList = nil); stdcall;

    {IGridItem interface}
    function getDisplayFontStyle: TFontStyles; stdcall;
    function getDisplayFontColor: TColor; stdcall;
    function getDateTime: TDateTime; stdcall;
    function getDisplayFlag: TGridFlag; stdcall;
    function getAlternateText: string; stdcall;

    {Observation functions}

    function getID: string; stdcall;
    function getParent: IObservation; stdcall;
    function getParentID: string; stdcall;
    function getBatteryID: string; stdcall;
    function getLocationID: string; stdcall;
    function getFacilityID: string; stdcall;

    function getPatientName: string; stdcall;
    function getPatientID: string; stdcall;

    function getObservedDateTime: TDateTime; stdcall;
    function getObservedBy: string; stdcall;
    function getObservedByID: string; stdcall;
    function getObservedByInitials: string; stdcall;

    function getEnteredDateTime: TDateTime; stdcall;
    function getEnteredBy: string; stdcall;
    function getEnteredByID: string; stdcall;

    function getObsTerm: ITerm; stdcall;
    function getObsUnit: ITerm; stdcall;

    function getSValue: string; stdcall;
    function getDisplayValue: string; stdcall;
    function getDisplayQualifiers: string; stdcall;
    function getNumericValue: double; stdcall;

    function getStatus: TObservationRecordStatus; stdcall;
    function getStatusValue: string; stdcall;

    function getRange: TObservationRange; stdcall;
    function getRangeValue: string; stdcall;

    function getSetCount: integer; stdcall;
    function getSetID(aIndex: integer): string; stdcall;

    function getComment: string; stdcall;

    function getHasAuditLog: boolean; stdcall;
    function getIsCorrectionForID: string; stdcall;

    function getSourceID: string; stdcall;
    function getSourceVersion: string; stdcall;
    function getSourceDataItemID: string; stdcall;
    function getSourceComments: string; stdcall;

    function getTerm(aTermType: TTermType): ITerm;
    function getTermID(aTermType: TTermType): string; stdcall;
    function getTermName(aTermType: TTermType): string; stdcall;
    function getTermDisplayName(aTermType: TTermType): string; stdcall;

    function getChildCount: integer; stdcall;
    function getChildID(aIndex: integer): string; stdcall;
    function getFirstChild: IObservation; stdcall;
    function getNextChild(aObservation: IObservation): IObservation; stdcall;

    function getHasAttribute(aID: string): boolean; stdcall;
    function IsPartOfSet(aSetID: string): boolean; stdcall;

    procedure addChildObservation(aObservation: IObservation); stdcall;
    procedure setParent(aObservation: IObservation); stdcall;

    function AddToListView(aListView: TListView): TListItem; stdcall;

    function isReadOnly: boolean; stdcall;

    function LoadFromXML(aXMLNode: IXMLNode; aTerminology: ITerminology): Boolean;
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewObservation(aXMLNode: IXMLNode; aTerminology: ITerminology; aReadOnly: boolean = false): IObservation;

implementation

uses
  uCPXML,
  StrUtils,
  Masks;

const
  FULL_DATE_FORMAT = 'mmmm dd, yyyy "at" hh:nn ';

function NewObservation(aXMLNode: IXMLNode; aTerminology: ITerminology; aReadOnly: boolean = false): IObservation;
begin
  with TObservation.Create do
    if LoadFromXML(aXMLNode, aTerminology) then
    begin
      GetInterface(IObservation, Result);
      fReadOnly := aReadOnly;
    end
    else
      Result := nil;
end;

{------------------------------------------------------------------------------}
{------------------------------ TObservation ----------------------------------}
{------------------------------------------------------------------------------}

function TObservation.AddToListView(aListView: TListView): TListItem;
begin
  Result := aListView.Items.Add;
  Result.Caption := fQualifiers[ttObservation].getTerm;
  Result.Data := Self;
end;

procedure TObservation.AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean; aList: ICliOList);
var
  i: integer;
  aValue: string;
  aWrapText: TStringList;
begin
  if getObsTerm.getDataType = dtNumeric then
    begin
      if getObsUnit <> nil then
        aValue := Format('%10.2f %-6s', [getNumericValue, '(' + Copy(getObsUnit.getAbbreviation, 1, 4) + ')'])
      else
        aValue := Format('%10.2f %-6s', [getNumericValue, '()']);
      aReportText.Add(
        Format('  %-20s%-8s%-15s%' + IntToStr(aMargin - 45) + 's',
        [FormatDateTime('yyyy-mm-dd hh:mm', getObservedDateTime), getObservedByInitials, getDisplayQualifiers, aValue]));
    end
  else
    try
      aWrapText := TStringList.Create;

      WrapThisValue(getDisplayValue, aMargin - 50, aWrapText);
      aReportText.Add(
        Format('  %-20s%-8s%-15s%-' + IntToStr(aMargin - 45) + 's',
        [FormatDateTime('yyyy-mm-dd hh:mm', getObservedDateTime), getObservedByInitials, getDisplayQualifiers, aWrapText.Strings[0]]));
      for i := 1 to aWrapText.Count - 1 do
        aReportText.Add(
          Format('  %-20s%-8s%-15s%-' + IntToStr(aMargin - 45) + 's',
          ['', '', '', aWrapText.Strings[i]]));
    finally
      aWrapText.Clear;
      FreeAndNil(aWrapText);
    end;

  if aIncludeDetail then
    if Length(getComment) > 0 then
      try
        aWrapText := TStringList.Create;
        WrapThisValue('Comment: ' + getComment, aMargin - 10, aWrapText);
        for i := 0 to aWrapText.Count - 1 do
          aReportText.Add('     ' + aWrapText.Strings[i]);
      finally
        aWrapText.Clear;
        FreeAndNil(aWrapText);
      end;
end;

constructor TObservation.Create;
var
  tt: TTermType;
begin
  inherited;
  TInterfaceList.Create.GetInterface(IInterfaceList, fChildren);
  fParent_ID := '';
  for tt := Low(TTermType) to High(TTermType) do
    fQualifiers[tt] := nil;
  fSValue := '';
  fSets := TStringList.Create;
  fNullFlavor := 0;
  fObserved_Date_Time := 0;
  fObserved_By_ID := '';
  fSource_Comments := '';
  fSource_DataItemID := '';
  fSource_ID := '';
  fSource_Version := '';
  fStatus_Record := osUnverified;
  fCorrection_For_ID := '';
  fAudit_Exists := False;
end;

destructor TObservation.Destroy;
var
  tt: TTermType;
begin
  for tt := Low(TTermType) to High(TTermType) do
    fQualifiers[tt] := nil;
  FreeAndNil(fSets);
  fChildren.Clear;
  fChildren := nil;
  inherited;
end;

function TObservation.getDateTime: TDateTime;
begin
  Result := fObserved_Date_Time;
end;

function TObservation.getDisplayValue: string;
var
  tt: TTermType;
  x: string;
  aQualifiers: string;
begin
  if fQualifiers[ttObservation].getDataType = dtBoolean then
    if VarToStrDef(fSValue, '0') = '1' then
      Result := fQualifiers[ttObservation].getBooleanValueTrue
    else
      Result := fQualifiers[ttObservation].getBooleanValueFalse
  else
    Result := fSValue;

  aQualifiers := '';
  for tt := Low(TTermType) to High(TTermType) do
    if tt in TERMQUALIFIERS then
      if fQualifiers[tt] <> nil then
        begin
          x := fQualifiers[tt].getAbbreviation;
          if x = '' then
            x := fQualifiers[tt].getTerm;
          aQualifiers := Format('%s%s ', [aQualifiers, x]);
        end;

  if aQualifiers <> '' then {Did we locate any qualifers?}
    Result := Format('%s (%s)', [Result, trim(aQualifiers)]);
end;

function TObservation.getDisplayQualifiers: string;
var
  tt: TTermType;
  x: string;
  aQualifiers: string;
begin
  aQualifiers := '';
  for tt := Low(TTermType) to High(TTermType) do
    if tt in [ttMethod, ttPosition, ttLocation, ttQuality, ttProduct, ttNullFlavor] then //TERMQUALIFIERS then
      if fQualifiers[tt] <> nil then
        begin
          x := fQualifiers[tt].getAbbreviation;
          if x = '' then
            x := fQualifiers[tt].getTerm;
          aQualifiers := Format('%s%s ', [aQualifiers, x]);
        end;

  if aQualifiers <> '' then {Did we locate any qualifers?}
    Result := trim(aQualifiers)
  else
    Result := '';
end;

function TObservation.getID: string;
begin
  Result := fID;
end;

function TObservation.getParent: IObservation;
begin
  Result := fParent;
end;

function TObservation.getParentID: string;
begin
  Result := fParent_ID;
end;

function TObservation.getTerm(aTermType: TTermType): ITerm;
begin
  Result := fQualifiers[aTermType];
end;

function TObservation.getTermID(aTermType: TTermType): string;
begin
  if fQualifiers[aTermType] <> nil then
    Result := fQualifiers[aTermType].getID
  else
    Result := '';
end;

function TObservation.IsPartOfSet(aSetID: string): boolean;
begin
  if aSetID <> '' then
    Result := (fSets.IndexOf(aSetID) > -1)
  else
    Result := True;
end;

function TObservation.isReadOnly: boolean;
begin
  {blj 5 Jan 2011 - Need to set this to ensure that the obs is read only if
    from vitals
  }
  result := fReadOnly or MatchesMask(fID,'GMRV*');
end;

function TObservation.getTermName(aTermType: TTermType): string;
begin
  if fQualifiers[aTermType] <> nil then
    Result := fQualifiers[aTermType].getTerm
  else
    Result := '';
end;

function TObservation.getTermDisplayName(aTermType: TTermType): string;
begin
  if fQualifiers[aTermType] <> nil then
    Result := fQualifiers[aTermType].getDisplayName
  else
    Result := '';
end;

function TObservation.LoadFromXML(aXMLNode: IXMLNode; aTerminology: ITerminology): Boolean;
var
  tt: TTermType;
  aNode: IXMLNode;
begin
  try
    {Basics to get it on the grid!}
    fID := XMLValueAsStr(aXMLNode, 'OBS_ID');
    fParent_ID := XMLValueAsStr(aXMLNode, 'PARENT_ID');
    fObserved_Date_Time := XMLValueAsDateTime(aXMLNode, 'OBSERVED_DATE_TIME');
    fSValue := XMLValueAsStr(aXMLNode, 'SVALUE');
    fStatus_Record := TObservationRecordStatus(XMLValueAsInt(aXMLNode, 'STATUS'));
    fRange := TObservationRange(XMLValueAsInt(aXMLNode, 'RANGE', Ord(orNormal)));
    fQualifiers[ttObservation] := aTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'TERM_ID'));
    fFacility_Location_ID := XMLValueAsStr(aXMLNode, 'FACILITY_LOCATION_ID');
    fHospital_Location_ID := XMLValueAsStr(aXMLNode, 'HOSPITAL_LOCATION_ID');

    {Load the qualifiers}
    for tt := Low(TTermType) to High(TTermType) do
      fQualifiers[tt] := aTerminology.GetTerm(XMLValueAsStr(aXMLNode, TERMTYPE_XMLTAG[tt]));

    {Convert Picklist ID to the display name}
    if (fQualifiers[ttObservation].getDataType = dtPicklist) and (fSValue <> '') then
      if Assigned(aTerminology.GetTerm(fSValue)) then
        fSValue := aTerminology.GetTerm(fSValue).getDisplayName;

    fPatient_ID := XMLValueAsStr(aXMLNode, 'PATIENT_ID');

    {Expanded list for inquiry type stuff}
    if fPatient_ID <> '' then
      begin
        //fBattery_ID := XMLValueAsStr(aXMLNode, 'BATTERY_ID');
        fPatient_Name := XMLValueAsStr(aXMLNode, 'PATIENT_NAME');

        fObserved_By := XMLValueAsStr(aXMLNode, 'OBSERVED_BY_NAME');
        fObserved_By_ID := XMLValueAsStr(aXMLNode, 'OBSERVED_BY_ID');
        fObserved_By_Initials := XMLValueAsStr(aXMLNode, 'OBSERVED_BY_INITIALS');

        fEntered_By := XMLValueAsStr(aXMLNode, 'ENTERED_BY');
        fEntered_By_ID := XMLValueAsStr(aXMLNode, 'ENTERED_BY_ID');
        fEntered_Date_Time := XMLValueAsDateTime(aXMLNode, 'ENTERED_DATE_TIME');

        fComment := XMLValueAsStr(aXMLNode, 'COMMENT');

        fSource_ID := XMLValueAsStr(aXMLNode, 'SOURCE');
        fSource_Comments := XMLValueAsStr(aXMLNode, 'SOURCE_COMMENTS');
        fSource_DataItemID := XMLValueAsStr(aXMLNode, 'SOURCE_DATA_ITEM_ID');
        fSource_Version := XMLValueAsStr(aXMLNode, 'SOURCE_VERSION');

        fCorrection_For_ID := XMLValueAsStr(aXMLNode, 'CORRECTION_FOR_ID');
        fAudit_Exists := (XMLValueAsInt(aXMLNode, 'AUDIT_EXISTS') = 1);

        {Load up any set IDs}
        aNode := aXMLNode.ChildNodes.FindNode('SETS');
        if Assigned(aNode) then
          begin
            aNode := aNode.ChildNodes.First;
            while Assigned(aNode) do
              try
                fSets.Add(aNode.ChildValues['VALUE']);
              finally
                aNode := aNode.NextSibling;
              end;
          end;
      end;
    Result := True;
  except {ReRaise any issues back to the calling program}
    raise;
  end;
end;

procedure TObservation.setParent(aObservation: IObservation);
begin
  if Assigned(aObservation) then
    begin
      fParent := aObservation;
      aObservation.addChildObservation(self);
    end;
end;

{
procedure TObservation.zzsetTerm(aTermType: TTermType; aTerm: TTerm);
begin
  fQualifiers[aTermType] := aTerm;
end;
}

procedure TObservation.addChildObservation(aObservation: IObservation);
begin
  fChildren.Add(aObservation);
end;

function TObservation.getSValue: string;
begin
  Result := VarToStrDef(fSValue, '');
end;

function TObservation.getChildCount: integer;
begin
  Result := fChildren.Count;
end;

function TObservation.getChildID(aIndex: integer): string;
begin
  if (aIndex < fChildren.Count) and (aIndex > -1) then
    Result := (fChildren[aIndex] as IObservation).getID
  else
    Result := '';
end;

function TObservation.getNumericValue: double;
begin
  Result := StrToFloatDef(VarToStrDef(fSValue, '0.0'), 0.0);
end;

function TObservation.getObsUnit: ITerm;
begin
  Result := fQualifiers[ttUnit];
end;

function TObservation.getObsTerm: ITerm;
begin
  Result := fQualifiers[ttObservation];
end;

function TObservation.getHasAttribute(aID: string): boolean;
var
  tt: TTermType;
begin
  if aID = '' then
    begin
      Result := True;
      Exit;
    end
  else
    for tt := Low(TTermType) to High(TTermType) do
      if Assigned(fQualifiers[tt]) then
        if fQualifiers[tt].getID = aID then
          begin
            Result := True;
            Exit; {Bail, we got a match!}
          end;
  {Nope!}
  Result := False;
end;

function TObservation.getStatus: TObservationRecordStatus;
begin
  Result := fStatus_Record;
end;

function TObservation.getStatusValue: string;
begin
  Result := OBSERVATION_STATUS[fStatus_Record];
end;

function TObservation.getFacilityID: string;
begin
  Result := fFacility_Location_ID;
end;

function TObservation.getFirstChild: IObservation;
begin
  if fChildren.Count > 0 then
    fChildren.First.QueryInterface(IObservation, Result)
  else
    Result := nil;
end;

function TObservation.getNextChild(aObservation: IObservation): IObservation;
var
  i: integer;
begin
  for i := 0 to fChildren.Count - 2 do
    if (fChildren[i] as IObservation).getID = aObservation.getID then
      begin
        fChildren[i + 1].QueryInterface(IObservation, Result);
        Exit;
      end;
  Result := nil;
end;

function TObservation.getDisplayFlag: TGridFlag;
begin
  if fComment <> '' then
    Result := gfInfo
  else
    Result := gfNone;
end;

function TObservation.getDisplayFontColor: TColor;
begin
  //Result := clWindowText;
  Result := clLime;
end;

function TObservation.getDisplayFontStyle: TFontStyles;
begin
  case fStatus_Record of
    osUnverified: Result := [fsItalic];
    osVerified: Result := [fsBold];
  else
    Result := [fsStrikeOut];
  end;
end;

function TObservation.getAlternateText: string;
var
  ObsUnit: string;
begin
  ObsUnit := '';
  if assigned(fQualifiers[ttUnit]) then
    ObsUnit := fQualifiers[ttUnit].getDescription;
  //
  Result := FSValue + ' ' +
            ObsUnit + ' observed at ' +
            FormatDateTime(FULL_DATE_FORMAT, FObserved_Date_Time) + ' recorded at ' +
            FormatDateTime(FULL_DATE_FORMAT, FEntered_Date_Time) + '.  ';
  //
  case fStatus_Record of
  osUnverified: result := result + ' Unverified. ';
  osVerified:  result := result + ' Verified. ';
  osArchived:  result := result + ' Archived. ';
  osPurged:  result := result + ' Purged. ';
  osCorrected:  result := result + ' Corrected. ';
  osRescinded:  result := result + ' Rescinded. ';
  end;

  // Now for some conditional stuff.
  case FRange of
  orUnknown: result := result + ' Range unknown. ' ;
  orOutOfBoundsLow: result := result + ' Range out of bounds low. ';
  orOutOfBoundsHigh: result := result + ' Range out of bounds high. ';
  orLow: result := result + ' Range low. ';
  orHigh: result := result + ' Range high. ';
  else
  end;

  if fAudit_Exists then
    result := result + ' Audit exists. ';

  if getChildCount > 0 then
    result := result + ' Observation has child obervations. ';
end;

function TObservation.getBatteryID: string;
begin
  Result := fBattery_ID;
end;

function TObservation.getEnteredBy: string;
begin
  Result := fEntered_By;
end;

function TObservation.getEnteredByID: string;
begin
  Result := fEntered_By_ID;
end;

function TObservation.getEnteredDateTime: TDateTime;
begin
  Result := fEntered_Date_Time;
end;

function TObservation.getObservedBy: string;
begin
  Result := fObserved_By;
end;

function TObservation.getObservedByID: string;
begin
  Result := fObserved_By_ID;
end;

function TObservation.getObservedByInitials: string;
begin
  Result := fObserved_By_Initials;
end;

function TObservation.getObservedDateTime: TDateTime;
begin
  Result := fObserved_Date_Time;
end;

function TObservation.getPatientID: string;
begin
  Result := fPatient_ID;
end;

function TObservation.getPatientName: string;
begin
  Result := fPatient_Name;
end;

function TObservation.getRange: TObservationRange;
begin
  Result := fRange;
end;

function TObservation.getRangeValue: string;
begin
  Result := OBSERVATION_RANGE[fRange];
end;

function TObservation.getSourceComments: string;
begin
  Result := fSource_Comments;
end;

function TObservation.getSourceDataItemID: string;
begin
  Result := fSource_DataItemID;
end;

function TObservation.getSourceID: string;
begin
  Result := fSource_ID;
end;

function TObservation.getSourceVersion: string;
begin
  Result := fSource_Version;
end;

function TObservation.getComment: string;
begin
  Result := fComment;
end;

function TObservation.getHasAuditLog: boolean;
begin
  Result := fAudit_Exists;
end;

function TObservation.getIsCorrectionForID: string;
begin
  Result := fCorrection_For_ID;
end;

function TObservation.getLocationID: string;
begin
  Result := fHospital_Location_ID;
end;

function TObservation.getSetCount: integer;
begin
  Result := fSets.Count;
end;

function TObservation.getSetID(aIndex: integer): string;
begin
  try
    Result := fSets[aIndex];
  except
    Result := '';
  end;
end;

end.

