unit oTerminology_Importer;

interface

uses
  Classes,
  Controls,
  Dialogs,
  oInterfaces,
  SysUtils,
  XMLDoc,
  XMLIntf;

type
  ETerminology_Exporter = Exception;

  TTerminology_Importer = class (TObject)
  private
    fDBConnection: IDBConnection;
    fDescriptionNode: IXMLNode;
    function ImportChildPairs(aXMLNode: IXMLNode): Boolean;
    function ImportQualifierPairs(aXMLNode: IXMLNode): Boolean;
    function ImportTerms(aXMLNode: IXMLNode): Boolean;
    function ImportUnitPairs(aXMLNode: IXMLNode): Boolean;
    function ImportValuePairs(aXMLNode: IXMLNode): Boolean;
  public
    function Execute(aFilename: string): Boolean;
    function GetFileDescription(aFilename: string; var aDescription: string): Boolean;
  end;
  
function NewTerminologyImporter(aDBConnection: IDBConnection): TTerminology_Importer;

implementation

uses
  oTerminology_Exporter;                          {Used only to get terminology version number}

function NewTerminologyImporter(aDBConnection: IDBConnection): TTerminology_Importer;
begin
  Result := TTerminology_Importer.Create;
  Result.fDBConnection := aDBConnection;
end;

{ TTerminologyImporter }

{
************************************************ TTerminology_Importer *************************************************
}
function TTerminology_Importer.Execute(aFilename: string): Boolean;
var
  aXMLDoc: IXMLDocument;
begin
  try
    aXMLDoc := NewXMLDocument;
    aXMLDoc.LoadFromFile(aFileName);
    aXMLDoc.Active := True;
  
    {Import Terms}
    ImportTerms(aXMLDoc.DocumentElement.ChildNodes.FindNode('TERMS'));
  
    {Import Child Pairs}
    ImportChildPairs(aXMLDoc.DocumentElement.ChildNodes.FindNode('CHILD_PAIRS'));
  
    {Import Qualifier Pairs}
    ImportQualifierPairs(aXMLDoc.DocumentElement.ChildNodes.FindNode('QUALIFIER_PAIRS'));
  
    {Import Unit Pairs}
    ImportUnitPairs(aXMLDoc.DocumentElement.ChildNodes.FindNode('UNIT_PAIRS'));
  
    {Import Value Pairs}
    ImportValuePairs(aXMLDoc.DocumentElement.ChildNodes.FindNode('PICKLIST_PAIRS'));
  
    Result := True;
  except
    Result := False;
  end;
end;

function TTerminology_Importer.GetFileDescription(aFilename: string; var aDescription: string): Boolean;
var
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
  aMessage: TStringList;
  aVersion: Double;
begin
  try
    aMessage := TStringList.Create;
  
    aXMLDoc := NewXMLDocument;
  
    aXMLDoc.LoadFromFile(aFileName);
  
    aXMLDoc.Active := True;
  
    aXMLNode := aXMLDoc.DocumentElement.ChildNodes.FindNode('DESCRIPTION');
    if aXMLNode <> nil then
      with aMessage do
        try
          Add(Format('Import ID: %s', [aXMLNode.ChildValues['ID']]));
          Add(Format('Date/Time Created: %s', [aXMLNode.ChildValues['DT_CREATED']]));
          Add(Format('Database Name: %s', [aXMLNode.ChildValues['DB_NAME']]));
          aVersion := aXMLNode.ChildValues['TERMINOLOGY_VERSION'];
          Add(Format('Terminology Version: %0.1f', [aVersion]));
          Add(Format('Number of Terms: %s', [aXMLNode.ChildValues['TERMS_EXPORTED']]));
          Add(' ');
        except
          raise ETerminology_Exporter.Create('Error Reading File Description Block.');
        end
    else
      raise ETerminology_Exporter.Create('File Description not found.');
  
    if aVersion < TERMINOLOGY_VERSION then        {Older version, should be backward compatible}
      begin
        aMessage.Add(Format(
          'WARNING: The import file is older (v%0.1f) than the loaded terminology (v%0.1f).',
          [aVersion, TERMINOLOGY_VERSION]));
        aMessage.Add(' ');
      end;
  
    if aVersion > TERMINOLOGY_VERSION then        {Newer version, can't make any guarantees, raise exception}
      raise ETerminology_Exporter.CreateFmt(
        'Error: The import file is newer (v%0.1f) than the loaded terminology (v%0.1f).',
        [aVersion, TERMINOLOGY_VERSION]);
  
    if aXMLDoc.DocumentElement.ChildNodes.FindNode('TERMS') = nil then
      raise ETerminology_Exporter.Create('No terms found');
  
    if aXMLDoc.DocumentElement.ChildNodes.FindNode('CHILD_PAIRS') = nil then
      raise ETerminology_Exporter.Create('No child pairs found');
  
    if aXMLDoc.DocumentElement.ChildNodes.FindNode('QUALIFIER_PAIRS') = nil then
      raise ETerminology_Exporter.Create('No qualifier pairs found');
  
    if aXMLDoc.DocumentElement.ChildNodes.FindNode('UNIT_PAIRS') = nil then
      raise ETerminology_Exporter.Create('No unit pairs found');
  
    if aXMLDoc.DocumentElement.ChildNodes.FindNode('PICKLIST_PAIRS') = nil then
      raise ETerminology_Exporter.Create('No value pairs found');
  
    aDescription := aMessage.Text;
    Result := True;
  except
    on E: ETerminology_Exporter do
      begin
        aDescription := e.Message;
        Result := False;
      end;
  else
    begin
      aDescription := 'Unknown file read error.';
      Result := False;
    end
  end;
end;

function TTerminology_Importer.ImportChildPairs(aXMLNode: IXMLNode): Boolean;
begin
  fDBConnection.ExecCommand('DeleteAllTermComplexPairs', []);
  aXMLNode := aXMLNode.ChildNodes.First;
  while aXMLNode <> nil do
    begin
      fDBConnection.ExecCommand('AddTermComplexPair', [
        aXMLNode.ChildValues['TERM_VUID'],
          aXMLNode.ChildValues['CHILD_VUID'],
          aXMLNode.ChildValues['ORDER']
          ]);
      aXMLNode := aXMLNode.NextSibling;
    end;
end;

function TTerminology_Importer.ImportQualifierPairs(aXMLNode: IXMLNode): Boolean;
begin
  fDBConnection.ExecCommand('DeleteAllTermQualifierPairs', []);
  aXMLNode := aXMLNode.ChildNodes.First;
  while aXMLNode <> nil do
    begin
      fDBConnection.ExecCommand('AddTermQualifierPair', [
        aXMLNode.ChildValues['TERM_VUID'],
          aXMLNode.ChildValues['QUAL_VUID']
          ]);
      aXMLNode := aXMLNode.NextSibling;
    end;
end;

function TTerminology_Importer.ImportTerms(aXMLNode: IXMLNode): Boolean;
begin
  fDBConnection.ExecCommand('DeleteAllTerms', []);
  aXMLNode := aXMLNode.ChildNodes.First;
  while aXMLNode <> nil do
    begin
      fDBConnection.ExecCommand('AddTerm', [
        aXMLNode.ChildValues['VUID'],
          aXMLNode.ChildValues['TERM_NAME'],
          aXMLNode.ChildValues['ABBREVIATION'],
          aXMLNode.ChildValues['DISPLAY_NAME'],
          aXMLNode.ChildValues['TERM_TYPE'],
          aXMLNode.ChildValues['DATA_TYPE'],
          aXMLNode.ChildValues['QUAL_CATEGORY'],
          aXMLNode.ChildValues['ACTIVE'],
          aXMLNode.ChildValues['DESCRIPTION'],
          aXMLNode.ChildValues['HELP_TEXT'],
          aXMLNode.ChildValues['VALUE_FORMAT']
          ]);
      aXMLNode := aXMLNode.NextSibling;
    end;
end;

function TTerminology_Importer.ImportUnitPairs(aXMLNode: IXMLNode): Boolean;
begin
  fDBConnection.ExecCommand('DeleteAllTermUnitPairs', []);
  aXMLNode := aXMLNode.ChildNodes.First;
  while aXMLNode <> nil do
    begin
      fDBConnection.ExecCommand('AddTermUnitPair', [
        aXMLNode.ChildValues['TERM_VUID'],
          aXMLNode.ChildValues['UNIT_VUID'],
          aXMLNode.ChildValues['MIN'],
          aXMLNode.ChildValues['MAX'],
          aXMLNode.ChildValues['DEC']
          ]);
      aXMLNode := aXMLNode.NextSibling;
    end;
end;

function TTerminology_Importer.ImportValuePairs(aXMLNode: IXMLNode): Boolean;
begin
  fDBConnection.ExecCommand('DeleteAllTermValuePairs', []);
  aXMLNode := aXMLNode.ChildNodes.First;
  while aXMLNode <> nil do
    begin
      fDBConnection.ExecCommand('AddTermValuePair', [
        aXMLNode.ChildValues['TERM_VUID'],
          aXMLNode.ChildValues['PICKLIST_VALUE'],
          aXMLNode.ChildValues['PICKLIST_ORDER']
          ]);
      aXMLNode := aXMLNode.NextSibling;
    end;
end;

end.

